@@ -11,7 +11,7 @@
 #'
 #' @param ip Object of class \code{character} representing the IP address of the server where H2O is running.
 #' @param port Object of class \code{numeric} representing the port number of the H2O server.
-#' @param name (Optional) A \code{character} string representing the H2O cloud name.
+#' @param name (Optional) A \code{character} string representing the H2O cluster name.
 #' @param startH2O (Optional) A \code{logical} value indicating whether to try to start H2O from R if no connection with H2O is detected. This is only possible if \code{ip = "localhost"} or \code{ip = "127.0.0.1"}.  If an existing connection is detected, R does not start H2O.
 #' @param forceDL (Optional) A \code{logical} value indicating whether to force download of the H2O executable. Defaults to FALSE, so the executable will only be downloaded if it does not already exist in the h2o R library resources directory \code{h2o/java/h2o.jar}.  This value is only used when R starts H2O.
 #' @param enable_assertions (Optional) A \code{logical} value indicating whether H2O should be launched with assertions enabled. Used mainly for error checking and debugging purposes.  This value is only used when R starts H2O.
@@ -38,7 +38,7 @@
 #' @note Users may wish to manually upgrade their package (rather than waiting until being prompted), which requires
 #' that they fully uninstall and reinstall the H2O package, and the H2O client package. You must unload packages running
 #' in the environment before upgrading. It's recommended that users restart R or R studio after upgrading
-#' @seealso \href{http://h2o-release.s3.amazonaws.com/h2o-dev/rel-shannon/2/docs-website/h2o-r/h2o_package.pdf}{H2O R package documentation} for more details. \code{\link{h2o.shutdown}} for shutting down from R.
+#' @seealso \href{http://docs.h2o.ai/h2o/latest-stable/h2o-r/h2o_package.pdf}{H2O R package documentation} for more details. \code{\link{h2o.shutdown}} for shutting down from R.
 #' @examples
 #' \dontrun{
 #' # Try to connect to a local H2O instance that is already running.
@@ -186,7 +186,7 @@ h2o.init <- function(ip = "localhost", port = 54321, name = NA_character_, start
                     max_memory = max_mem_size, min_memory = min_mem_size,
                     enable_assertions = enable_assertions, forceDL = forceDL, license = license,
                     extra_classpath = extra_classpath, ice_root = ice_root, stdout = stdout,
-                    log_dir = log_dir, log_level = log_level,
+                    log_dir = log_dir, log_level = log_level, context_path = context_path,
                     jvm_custom_args = jvm_custom_args, bind_to_localhost = bind_to_localhost)
 
       count <- 0L
@@ -353,7 +353,7 @@ h2o.getConnection <- function() {
 #' @note Users must call h2o.shutdown explicitly in order to shut down the local H2O instance started by R. If R is closed before H2O, then an attempt will be made to automatically shut down H2O. This only applies to local instances started with h2o.init, not remote H2O servers.
 #' @seealso \code{\link{h2o.init}}
 #' @examples
-#' # Don't run automatically to prevent accidentally shutting down a cloud
+#' # Don't run automatically to prevent accidentally shutting down a cluster
 #' \dontrun{
 #' library(h2o)
 #' h2o.init()
@@ -387,7 +387,7 @@ h2o.shutdown <- function(prompt = TRUE) {
 # **** TODO: This isn't really a cluster status... it's a node status check for the node we're connected to.
 # This is possibly confusing because this can come back without warning,
 # but if a user tries to do any remoteSend, they will get a "cloud sick warning"
-# Suggest cribbing the code from Internal.R that checks cloud status (or just call it here?)
+# Suggest cribbing the code from Internal.R that checks cluster status (or just call it here?)
 
 #' Return the status of the cluster
 #'
@@ -395,7 +395,7 @@ h2o.shutdown <- function(prompt = TRUE) {
 #'
 #' @seealso \linkS4class{H2OConnection}, \code{\link{h2o.init}}
 #' @examples
-#' \donttest{
+#' \dontrun{
 #' h2o.init()
 #' h2o.clusterStatus()
 #' }
@@ -530,7 +530,8 @@ h2o.clusterStatus <- function() {
 .h2o.startJar <- function(ip = "localhost", port = 54321, name = NULL, nthreads = -1,
                           max_memory = NULL, min_memory = NULL,
                           enable_assertions = TRUE, forceDL = FALSE, license = NULL, extra_classpath = NULL,
-                          ice_root, stdout, log_dir, log_level, jvm_custom_args = NULL, bind_to_localhost) {
+                          ice_root, stdout, log_dir, log_level, context_path, jvm_custom_args = NULL, 
+                          bind_to_localhost) {
   command <- .h2o.checkJava()
 
   if (! is.null(license)) {
@@ -612,6 +613,7 @@ h2o.clusterStatus <- function() {
 
   if(!is.na(log_dir)) args <- c(args, "-log_dir", log_dir)
   if(!is.na(log_level)) args <- c(args, "-log_level", log_level)
+  if(!is.na(context_path)) args <- c(args, "-context_path", context_path)
 
   if(nthreads > 0L) args <- c(args, "-nthreads", nthreads)
   if(!is.null(license)) args <- c(args, "-license", license)
@@ -822,6 +824,8 @@ h2o.networkTest <- function() {
 #'
 #' @importFrom utils browseURL
 #' @export
-h2o.flow <- function(){
-  browseURL(.h2o.calcBaseURL(urlSuffix=""))
+h2o.flow <- function() {
+  conn <- h2o.getConnection()
+  url <- .h2o.calcBaseURL(conn, urlSuffix = "flow/")
+  browseURL(url)
 }
